@@ -56,7 +56,7 @@ typedef struct
 	BOOL mode;
 } RFX_PROGRESSIVE_UPGRADE_STATE;
 
-static INLINE void
+static inline void
 progressive_component_codec_quant_read(wStream* WINPR_RESTRICT s,
                                        RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT quantVal)
 {
@@ -78,7 +78,7 @@ progressive_component_codec_quant_read(wStream* WINPR_RESTRICT s,
 	quantVal->HH1 = b >> 4;
 }
 
-static INLINE void progressive_rfx_quant_add(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
+static inline void progressive_rfx_quant_add(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
                                              const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q2,
                                              RFX_COMPONENT_CODEC_QUANT* dst)
 {
@@ -94,7 +94,7 @@ static INLINE void progressive_rfx_quant_add(const RFX_COMPONENT_CODEC_QUANT* WI
 	dst->LL3 = q1->LL3 + q2->LL3; /* LL3 */
 }
 
-static INLINE void progressive_rfx_quant_lsub(RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q, int val)
+static inline void progressive_rfx_quant_lsub(RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q, int val)
 {
 	q->HL1 -= val; /* HL1 */
 	q->LH1 -= val; /* LH1 */
@@ -108,7 +108,7 @@ static INLINE void progressive_rfx_quant_lsub(RFX_COMPONENT_CODEC_QUANT* WINPR_R
 	q->LL3 -= val; /* LL3 */
 }
 
-static INLINE void progressive_rfx_quant_sub(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
+static inline void progressive_rfx_quant_sub(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
                                              const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q2,
                                              RFX_COMPONENT_CODEC_QUANT* dst)
 {
@@ -124,7 +124,7 @@ static INLINE void progressive_rfx_quant_sub(const RFX_COMPONENT_CODEC_QUANT* WI
 	dst->LL3 = q1->LL3 - q2->LL3; /* LL3 */
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_rfx_quant_lcmp_less_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q, int val)
 {
 	if (q->HL1 > val)
@@ -160,7 +160,7 @@ progressive_rfx_quant_lcmp_less_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RES
 	return TRUE;
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_rfx_quant_lcmp_greater_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q, int val)
 {
 	if (q->HL1 < val)
@@ -196,7 +196,7 @@ progressive_rfx_quant_lcmp_greater_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_
 	return TRUE;
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_rfx_quant_cmp_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
                                 const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q2)
 {
@@ -233,7 +233,7 @@ progressive_rfx_quant_cmp_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT
 	return TRUE;
 }
 
-static INLINE BOOL progressive_set_surface_data(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline BOOL progressive_set_surface_data(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                 UINT16 surfaceId,
                                                 PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT pData)
 {
@@ -247,7 +247,7 @@ static INLINE BOOL progressive_set_surface_data(PROGRESSIVE_CONTEXT* WINPR_RESTR
 	return TRUE;
 }
 
-static INLINE PROGRESSIVE_SURFACE_CONTEXT*
+static inline PROGRESSIVE_SURFACE_CONTEXT*
 progressive_get_surface_data(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, UINT16 surfaceId)
 {
 	void* key = (void*)(((ULONG_PTR)surfaceId) + 1);
@@ -290,7 +290,7 @@ static void progressive_surface_context_free(void* ptr)
 	winpr_aligned_free(surface);
 }
 
-static INLINE RFX_PROGRESSIVE_TILE* progressive_tile_new(void)
+static inline RFX_PROGRESSIVE_TILE* progressive_tile_new(void)
 {
 	RFX_PROGRESSIVE_TILE* tile = winpr_aligned_calloc(1, sizeof(RFX_PROGRESSIVE_TILE), 32);
 	if (!tile)
@@ -323,7 +323,7 @@ fail:
 	return NULL;
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_allocate_tile_cache(PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface, size_t min)
 {
 	size_t oldIndex = 0;
@@ -387,7 +387,7 @@ static PROGRESSIVE_SURFACE_CONTEXT* progressive_surface_context_new(UINT16 surfa
 	return surface;
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_surface_tile_replace(PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
                                  PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
                                  const RFX_PROGRESSIVE_TILE* WINPR_RESTRICT tile, BOOL upgrade)
@@ -527,7 +527,7 @@ static int16_t clampi16(int val)
 	return (int16_t)val;
 }
 
-static INLINE void progressive_rfx_idwt_x(const INT16* WINPR_RESTRICT pLowBand, size_t nLowStep,
+static inline void progressive_rfx_idwt_x(const INT16* WINPR_RESTRICT pLowBand, size_t nLowStep,
                                           const INT16* WINPR_RESTRICT pHighBand, size_t nHighStep,
                                           INT16* WINPR_RESTRICT pDstBand, size_t nDstStep,
                                           size_t nLowCount, size_t nHighCount, size_t nDstCount)
@@ -596,7 +596,7 @@ static INLINE void progressive_rfx_idwt_x(const INT16* WINPR_RESTRICT pLowBand,
 	}
 }
 
-static INLINE void progressive_rfx_idwt_y(const INT16* WINPR_RESTRICT pLowBand, size_t nLowStep,
+static inline void progressive_rfx_idwt_y(const INT16* WINPR_RESTRICT pLowBand, size_t nLowStep,
                                           const INT16* WINPR_RESTRICT pHighBand, size_t nHighStep,
                                           INT16* WINPR_RESTRICT pDstBand, size_t nDstStep,
                                           size_t nLowCount, size_t nHighCount, size_t nDstCount)
@@ -671,12 +671,12 @@ static INLINE void progressive_rfx_idwt_y(const INT16* WINPR_RESTRICT pLowBand,
 	}
 }
 
-static INLINE size_t progressive_rfx_get_band_l_count(size_t level)
+static inline size_t progressive_rfx_get_band_l_count(size_t level)
 {
 	return (64 >> level) + 1;
 }
 
-static INLINE size_t progressive_rfx_get_band_h_count(size_t level)
+static inline size_t progressive_rfx_get_band_h_count(size_t level)
 {
 	if (level == 1)
 		return (64 >> 1) - 1;
@@ -684,7 +684,7 @@ static INLINE size_t progressive_rfx_get_band_h_count(size_t level)
 		return (64 + (1 << (level - 1))) >> level;
 }
 
-static INLINE void progressive_rfx_dwt_2d_decode_block(INT16* WINPR_RESTRICT buffer,
+static inline void progressive_rfx_dwt_2d_decode_block(INT16* WINPR_RESTRICT buffer,
                                                        INT16* WINPR_RESTRICT temp, size_t level)
 {
 	size_t nDstStepX = 0;
@@ -736,7 +736,7 @@ void rfx_dwt_2d_extrapolate_decode(INT16* WINPR_RESTRICT buffer, INT16* WINPR_RE
 	progressive_rfx_dwt_2d_decode_block(&buffer[0], temp, 1);
 }
 
-static INLINE int progressive_rfx_dwt_2d_decode(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline int progressive_rfx_dwt_2d_decode(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                 INT16* WINPR_RESTRICT buffer,
                                                 INT16* WINPR_RESTRICT current, BOOL coeffDiff,
                                                 BOOL extrapolate, BOOL reverse)
@@ -773,7 +773,7 @@ static INLINE int progressive_rfx_dwt_2d_decode(PROGRESSIVE_CONTEXT* WINPR_RESTR
 	return 1;
 }
 
-static INLINE void progressive_rfx_decode_block(const primitives_t* prims,
+static inline void progressive_rfx_decode_block(const primitives_t* prims,
                                                 INT16* WINPR_RESTRICT buffer, UINT32 length,
                                                 UINT32 shift)
 {
@@ -783,7 +783,7 @@ static INLINE void progressive_rfx_decode_block(const primitives_t* prims,
 	prims->lShiftC_16s_inplace(buffer, shift, length);
 }
 
-static INLINE int
+static inline int
 progressive_rfx_decode_component(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                  const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT shift,
                                  const BYTE* WINPR_RESTRICT data, UINT32 length,
@@ -832,7 +832,7 @@ progressive_rfx_decode_component(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive
 	                                     FALSE);
 }
 
-static INLINE int
+static inline int
 progressive_decompress_tile_first(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                   RFX_PROGRESSIVE_TILE* WINPR_RESTRICT tile,
                                   PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
@@ -973,7 +973,7 @@ fail:
 	return rc;
 }
 
-static INLINE INT16 progressive_rfx_srl_read(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state,
+static inline INT16 progressive_rfx_srl_read(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state,
                                              UINT32 numBits)
 {
 	WINPR_ASSERT(state);
@@ -1062,7 +1062,7 @@ static INLINE INT16 progressive_rfx_srl_read(RFX_PROGRESSIVE_UPGRADE_STATE* WINP
 	return (INT16)(sign ? -1 * (int)mag : (INT16)mag);
 }
 
-static INLINE int
+static inline int
 progressive_rfx_upgrade_state_finish(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state)
 {
 	UINT32 pad = 0;
@@ -1090,7 +1090,7 @@ progressive_rfx_upgrade_state_finish(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRI
 	return 1;
 }
 
-static INLINE int progressive_rfx_upgrade_block(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state,
+static inline int progressive_rfx_upgrade_block(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state,
                                                 INT16* WINPR_RESTRICT buffer,
                                                 INT16* WINPR_RESTRICT sign, UINT32 length,
                                                 UINT32 shift, WINPR_ATTR_UNUSED UINT32 bitPos,
@@ -1151,7 +1151,7 @@ static INLINE int progressive_rfx_upgrade_block(RFX_PROGRESSIVE_UPGRADE_STATE* W
 	return 1;
 }
 
-static INLINE int progressive_rfx_upgrade_component(
+static inline int progressive_rfx_upgrade_component(
     PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
     const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT shift,
     const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT bitPos,
@@ -1249,7 +1249,7 @@ static INLINE int progressive_rfx_upgrade_component(
 	                                     TRUE);
 }
 
-static INLINE int
+static inline int
 progressive_decompress_tile_upgrade(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                     RFX_PROGRESSIVE_TILE* WINPR_RESTRICT tile,
                                     PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
@@ -1425,7 +1425,7 @@ fail:
 	return status;
 }
 
-static INLINE BOOL progressive_tile_read_upgrade(
+static inline BOOL progressive_tile_read_upgrade(
     PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, wStream* WINPR_RESTRICT s, UINT16 blockType,
     UINT32 blockLen, PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
     PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
@@ -1503,11 +1503,12 @@ static INLINE BOOL progressive_tile_read_upgrade(
 	return progressive_surface_tile_replace(surface, region, &tile, TRUE);
 }
 
-static INLINE BOOL progressive_tile_read(
-    PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, BOOL simple, wStream* WINPR_RESTRICT s,
-    UINT16 blockType, UINT32 blockLen, PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
-    PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
-    WINPR_ATTR_UNUSED const PROGRESSIVE_BLOCK_CONTEXT* WINPR_RESTRICT context)
+static inline BOOL
+progressive_tile_read(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, BOOL simple,
+                      wStream* WINPR_RESTRICT s, UINT16 blockType, UINT32 blockLen,
+                      PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
+                      PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
+                      WINPR_ATTR_UNUSED const PROGRESSIVE_BLOCK_CONTEXT* WINPR_RESTRICT context)
 {
 	RFX_PROGRESSIVE_TILE tile = { 0 };
 	size_t expect = simple ? 16 : 17;
@@ -1593,11 +1594,12 @@ static void CALLBACK progressive_process_tiles_tile_work_callback(PTP_CALLBACK_I
 	}
 }
 
-static INLINE SSIZE_T progressive_process_tiles(
-    PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, wStream* WINPR_RESTRICT s,
-    PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
-    PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
-    const PROGRESSIVE_BLOCK_CONTEXT* WINPR_RESTRICT context)
+static inline SSIZE_T
+progressive_process_tiles(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+                          wStream* WINPR_RESTRICT s,
+                          PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
+                          PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
+                          const PROGRESSIVE_BLOCK_CONTEXT* WINPR_RESTRICT context)
 {
 	int status = 0;
 	size_t end = 0;
@@ -1742,7 +1744,7 @@ fail:
 	return (SSIZE_T)(end - start);
 }
 
-static INLINE SSIZE_T progressive_wb_sync(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_sync(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                           wStream* WINPR_RESTRICT s, UINT16 blockType,
                                           UINT32 blockLen)
 {
@@ -1794,7 +1796,7 @@ static INLINE SSIZE_T progressive_wb_sync(PROGRESSIVE_CONTEXT* WINPR_RESTRICT pr
 	return 0;
 }
 
-static INLINE SSIZE_T progressive_wb_frame_begin(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_frame_begin(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                  wStream* WINPR_RESTRICT s, UINT16 blockType,
                                                  UINT32 blockLen)
 {
@@ -1846,7 +1848,7 @@ static INLINE SSIZE_T progressive_wb_frame_begin(PROGRESSIVE_CONTEXT* WINPR_REST
 	return 0;
 }
 
-static INLINE SSIZE_T progressive_wb_frame_end(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_frame_end(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                wStream* WINPR_RESTRICT s, UINT16 blockType,
                                                UINT32 blockLen)
 {
@@ -1884,7 +1886,7 @@ static INLINE SSIZE_T progressive_wb_frame_end(PROGRESSIVE_CONTEXT* WINPR_RESTRI
 	return 0;
 }
 
-static INLINE SSIZE_T progressive_wb_context(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_context(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                              wStream* WINPR_RESTRICT s, UINT16 blockType,
                                              UINT32 blockLen)
 {
@@ -1932,9 +1934,10 @@ static INLINE SSIZE_T progressive_wb_context(PROGRESSIVE_CONTEXT* WINPR_RESTRICT
 	return 0;
 }
 
-static INLINE SSIZE_T progressive_wb_read_region_header(
-    PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, wStream* WINPR_RESTRICT s, UINT16 blockType,
-    UINT32 blockLen, PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region)
+static inline SSIZE_T
+progressive_wb_read_region_header(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+                                  wStream* WINPR_RESTRICT s, UINT16 blockType, UINT32 blockLen,
+                                  PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region)
 {
 	region->usedTiles = 0;
 
@@ -2019,7 +2022,7 @@ static INLINE SSIZE_T progressive_wb_read_region_header(
 	return rc;
 }
 
-static INLINE SSIZE_T progressive_wb_skip_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_skip_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                  wStream* WINPR_RESTRICT s, UINT16 blockType,
                                                  UINT32 blockLen)
 {
@@ -2036,7 +2039,7 @@ static INLINE SSIZE_T progressive_wb_skip_region(PROGRESSIVE_CONTEXT* WINPR_REST
 	return rc;
 }
 
-static INLINE SSIZE_T progressive_wb_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                             wStream* WINPR_RESTRICT s, UINT16 blockType,
                                             UINT32 blockLen,
                                             PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
@@ -2157,7 +2160,7 @@ static INLINE SSIZE_T progressive_wb_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT
 	return rc;
 }
 
-static INLINE SSIZE_T progressive_parse_block(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_parse_block(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                               wStream* WINPR_RESTRICT s,
                                               PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
                                               PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region)
@@ -2226,7 +2229,7 @@ static INLINE SSIZE_T progressive_parse_block(PROGRESSIVE_CONTEXT* WINPR_RESTRIC
 	return rc;
 }
 
-static INLINE BOOL update_tiles(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline BOOL update_tiles(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                 PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
                                 BYTE* WINPR_RESTRICT pDstData, UINT32 DstFormat, UINT32 nDstStep,
                                 UINT32 nXDst, UINT32 nYDst,
