@@ -18,9 +18,7 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <freerdp/config.h>
 
 #include "shadow.h"
 
@@ -28,12 +26,12 @@
 
 int shadow_client_remdesk_init(rdpShadowClient* client)
 {
-	RemdeskServerContext* remdesk;
+	RemdeskServerContext* remdesk = NULL;
 
 	remdesk = client->remdesk = remdesk_server_context_new(client->vcm);
 	remdesk->rdpcontext = &client->context;
 
-	remdesk->custom = (void*) client;
+	remdesk->custom = (void*)client;
 
 	if (client->remdesk)
 		client->remdesk->Start(client->remdesk);
@@ -43,7 +41,8 @@ int shadow_client_remdesk_init(rdpShadowClient* client)
 
 void shadow_client_remdesk_uninit(rdpShadowClient* client)
 {
-	if (client->remdesk) {
+	if (client->remdesk)
+	{
 		client->remdesk->Stop(client->remdesk);
 		remdesk_server_context_free(client->remdesk);
 		client->remdesk = NULL;
